/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.epms.service.impl;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.rmi.RemoteException;

import javax.xml.rpc.ServiceException;

import kgrid.epms.service.impl.exception.SystemPermissionException;
import kgrid.epms.service.impl.utils.Security;
import kgrid.epms.service.impl.utils.Session;
import kgrid.epms.stubs.EPMSService.Parameters;
import kgrid.raems.stubs.RAEMSFactory.CreateResource;
import kgrid.raems.stubs.RAEMSFactory.CreateResourceResponse;
import kgrid.raems.stubs.RAEMSFactory.RAEMSFactoryPortType;
import kgrid.raems.stubs.RAEMSFactory.service.RAEMSFactoryServiceAddressingLocator;
import kgrid.raems.stubs.RAEMSService.ExecutionParameters;
import kgrid.raems.stubs.RAEMSService.RAEMSPortType;
import kgrid.raems.stubs.RAEMSService.service.RAEMSServiceAddressingLocator;

import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.types.URI.MalformedURIException;
import org.apache.xml.serialize.DOMSerializer;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.globus.wsrf.NoSuchResourceException;
import org.globus.wsrf.ResourceContext;
import org.w3c.dom.Document;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class EPMSService implements Consts {

	private Session session = null;

	/**
	 * Gets a reference to the resource specified in the endpoint reference.
	 * 
	 * @throws RemoteException
	 */
	private EPMSResource getResource() throws RemoteException {
		Object resource = null;
		try {
			resource = ResourceContext.getResourceContext().getResource();
		} catch (Exception e) {
			throw new RemoteException("", e);
		}

		EPMSResource EPMSResource = (EPMSResource) resource;
		return EPMSResource;
	}

	public ExecutionParameters submitKApplication(Parameters parameters)
			throws RemoteException {
		try {
			EndpointReferenceType eprCredential = parameters.getEprCredential();
			createSession(eprCredential);
			Document conceptualModel = null;
			EPMSResource resource = getResource();
			if (resource == null) {
				String message = "#ERROR: Resource not found";
				Utils.print(message);
				return null;
			}
			if (parameters.getConceptualModel() instanceof Document) {
				conceptualModel = parameters.getConceptualModel();
				resource.setModel(conceptualModel);
				// Conceptual model received. Processing started...
			} else {
				String message = "#ERROR: Conceptual model not received";
				resource.setLastOp(message);
				Utils.print(message);
				return null;
			}
			EPMSTranslator epmsTranslate = new EPMSTranslator();
			Document executionPlanDoc = epmsTranslate
					.getExecutionPlan(conceptualModel);
			if (executionPlanDoc == null) {
				String message = "#ERROR: Translation failed";
				resource.setLastOp(message);
				Utils.print(message);
				return null;
			}
			resource.setPlan(executionPlanDoc);
			ExecutionParameters ep = new ExecutionParameters();
			ep.setEprCredential(session.getClientCredentials());
			ep.setExecutionPlanDoc(executionPlanDoc);
			// Abstract plan successfully created
			RAEMSFactoryServiceAddressingLocator factoryLocator = new RAEMSFactoryServiceAddressingLocator();
			RAEMSServiceAddressingLocator instanceLocator = new RAEMSServiceAddressingLocator();
			String raemsFactoryURI = "https://" + Utils.HOST_IP + ":"
					+ CONTAINER_PORT + "/" + RAEMS_FACTORY;
			EndpointReferenceType factoryRaemsEPR, instanceRaemsEPR;
			RAEMSFactoryPortType raemsFactory;
			RAEMSPortType raems;
			// 1: get Factory Port Type
			factoryRaemsEPR = new EndpointReferenceType();
			factoryRaemsEPR.setAddress(new Address(raemsFactoryURI));
			raemsFactory = factoryLocator
					.getRAEMSFactoryPortTypePort(factoryRaemsEPR);
			// 2: Create resource and get endpoint reference of WS-Resource.
			// This resource is our "instance".
			CreateResourceResponse createResponse = raemsFactory
					.createResource(new CreateResource());
			instanceRaemsEPR = createResponse.getEndpointReference();
			raems = instanceLocator.getRAEMSPortTypePort(instanceRaemsEPR);
			EPMSListener epmsML = new EPMSListener(raems, resource,
					epmsTranslate.getTaskNodesMapping(),
					epmsTranslate.getIdNodeMapping(),
					epmsTranslate.getInfoEdge());
			try {
				saveDocument(executionPlanDoc,
						SESSION_PATH + session.getSessionID()
								+ "/abstractExecutionPlan.xml");
			} catch (IOException e) {
				e.printStackTrace();
			}
			raems.manageKExecution(ep);
			return ep;
		} catch (RemoteException e) {
			Utils.print(e.getMessage());
		} catch (MalformedURIException e) {
			e.printStackTrace();
		} catch (ServiceException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Save a org.w3c.dom.Document object on a specified file
	 * 
	 * @param doc
	 * @param fileName
	 * @throws IOException
	 */
	private void saveDocument(Document doc, String fileName) throws IOException {
		String finalFileName = "";
		if (fileName.endsWith(".xml"))
			finalFileName = fileName;
		else
			finalFileName = fileName + ".xml";
		PrintWriter pw = new PrintWriter(new FileWriter(finalFileName));
		DOMSerializer ser = new XMLSerializer(pw, new OutputFormat(doc,
				"UTF-8", true));
		ser.serialize(doc);
	}

	Session getSession() {
		return session;
	}

	/*
	 * Create a new session which consists of a directory in the KEPR repository
	 * named with the current time in milliseconds
	 */
	private void createSession(EndpointReferenceType eprCredential)
			throws SystemPermissionException, NoSuchResourceException {
		File f = null;
		try {
			String sessionID = "" + System.currentTimeMillis() + "/";
			Utils.print("Instanciating the new resource");
			session = new Session(getResource());
			Utils.print("Setting the session id " + sessionID);
			session.setSessionID(sessionID);
			Utils.print("Setting the credentials");
			session.setClientCredentials(Security.delegateCredential(
					Utils.HOST_IP, CONTAINER_PORT, eprCredential));
			Utils.print("Creating the new file " + SESSION_PATH
					+ session.getSessionID());
			f = new File(SESSION_PATH + session.getSessionID());
			Utils.print("Creating the new directory " + SESSION_PATH
					+ session.getSessionID());
			f.mkdirs();
			Utils.print("Directory created");
		} catch (NoSuchResourceException ex) {
			throw ex;
		} catch (Exception ex) {
			throw new SystemPermissionException("Cannot create folder "
					+ session + "\n" + ex.getMessage());
		}
		if (!f.setWritable(true, false)) {
			throw new SystemPermissionException(
					"Cannot change folder permission: +w");
		}
		if (!f.setReadable(true, false)) {
			throw new SystemPermissionException(
					"Cannot change folder permission: +r");
		}

	}
}
