/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.epms.service.impl;

import java.util.HashMap;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import kgrid.epms.service.impl.tag.Access;
import kgrid.epms.service.impl.tag.Arg;
import kgrid.epms.service.impl.tag.Args;
import kgrid.epms.service.impl.tag.Attribute;
import kgrid.epms.service.impl.tag.AttributeSeparatorString;
import kgrid.epms.service.impl.tag.Attributes;
import kgrid.epms.service.impl.tag.Computation;
import kgrid.epms.service.impl.tag.DMSoftInput;
import kgrid.epms.service.impl.tag.DataMiningSoftware;
import kgrid.epms.service.impl.tag.DataTransfer;
import kgrid.epms.service.impl.tag.Dataset;
import kgrid.epms.service.impl.tag.Description;
import kgrid.epms.service.impl.tag.DocumentationURL;
import kgrid.epms.service.impl.tag.DrivingMethod;
import kgrid.epms.service.impl.tag.Edge;
import kgrid.epms.service.impl.tag.EdgeMapping;
import kgrid.epms.service.impl.tag.End;
import kgrid.epms.service.impl.tag.FinalTagEP;
import kgrid.epms.service.impl.tag.Format;
import kgrid.epms.service.impl.tag.HeaderEP;
import kgrid.epms.service.impl.tag.HostName;
import kgrid.epms.service.impl.tag.Input;
import kgrid.epms.service.impl.tag.Invocation;
import kgrid.epms.service.impl.tag.KindOfData;
import kgrid.epms.service.impl.tag.KindOfKnowledge;
import kgrid.epms.service.impl.tag.KindOfSoftware;
import kgrid.epms.service.impl.tag.KindOfTechnique;
import kgrid.epms.service.impl.tag.Location;
import kgrid.epms.service.impl.tag.ManualPath;
import kgrid.epms.service.impl.tag.Model;
import kgrid.epms.service.impl.tag.Output;
import kgrid.epms.service.impl.tag.OutputFolder;
import kgrid.epms.service.impl.tag.Parameter;
import kgrid.epms.service.impl.tag.Program;
import kgrid.epms.service.impl.tag.RecordSeparatorString;
import kgrid.epms.service.impl.tag.Size;
import kgrid.epms.service.impl.tag.Start;
import kgrid.epms.service.impl.tag.Structure;
import kgrid.epms.service.impl.tag.SubType;
import kgrid.epms.service.impl.tag.Task;
import kgrid.epms.service.impl.tag.TaskLink;
import kgrid.epms.service.impl.tag.UnknownTokenString;
import kgrid.epms.service.impl.tag.Usage;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * 
 * This class contains the methods used for the translation of the conceptual
 * model into the abstract execution plan. It makes two assumptions about the
 * conceptual model:
 * 
 * 1. all the node elements must precede all the edge elements;
 * 
 * 2. all the edge elements must have ids numbered progressively from the root
 * nodes to the leaf nodes; that is, the id of an edge in a chain of edges must
 * be greater than all the ids of its preceding edges.
 * 
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class EPMSTranslator implements Consts {
	private HashMap<String, Object> nodesMapping = null;
	private HashMap<String, Vector> mapTaskNode = null;
	private Vector<Edge> edges = null;
	private Vector<String> listEdgeTransfer = null;
	private Vector<TaskLink> taskLinks = null;
	private Vector<String> listTaskID = null;
	private Vector<String> leafTaskLinks = null;
	private int numTask, numInput, numOutput, numProgram, numTransfer;
	private String epmsPlan = "";
	// numero di trasferimenti per task
	private HashMap<String, Integer> transferCount;
	private HashMap<String, Integer> transferCountInput;
	// mapping (nomeTask:IDresource),edge;
	// utile per la ricezione delle notifiche dei rami
	private HashMap<String, Edge> infoEdge;

	private Vector<String> mappedNodes = null;

	private void initialize() {
		nodesMapping = new HashMap<String, Object>();
		mapTaskNode = new HashMap<String, Vector>();
		edges = new Vector<Edge>();
		listEdgeTransfer = new Vector<String>();
		taskLinks = new Vector<TaskLink>();
		listTaskID = new Vector<String>();
		leafTaskLinks = new Vector<String>();
		numTask = 0;
		numInput = 0;
		numOutput = 0;
		numProgram = 0;
		numTransfer = 0;
		epmsPlan = "";
		transferCount = new HashMap<String, Integer>();
		transferCountInput = new HashMap<String, Integer>();
		infoEdge = new HashMap<String, Edge>();
		mappedNodes = new Vector<String>();
	}

	public Document getExecutionPlan(Document conceptualModel) {
		try {
			initialize();
			parse(conceptualModel);
			for (int i = 0; i < listTaskID.size(); i++) {
				String id = listTaskID.get(i);
				Object o = nodesMapping.get(id);
				if (o instanceof Task) {
					Task task = (Task) o;
					epmsPlan += task.toString() + "\n";
				}
			}
			epmsPlan += End.string() + "\n";
			for (int i = 0; i < taskLinks.size(); i++) {
				TaskLink tl = taskLinks.get(i);
				epmsPlan += tl + "\n";
				/*
				 * I must find the tasklink targets which are not sources of any
				 * other tasklink (i.e. the leaves): they will become sources of
				 * new tasklinks with END as target
				 */
				String sourceJob = tl.getFrom();
				String targetJob = tl.getTo();
				if (sourceJob == null || targetJob == null) {
					throw new Exception("Execution plan generation error at "
							+ tl.toString());
				}
				if (!sourceJob.equals("START")) {
					if (leafTaskLinks.contains(sourceJob)) {
						leafTaskLinks.remove(sourceJob);
					}
				}
				leafTaskLinks.add(targetJob);
			}
			for (int i = 0; i < leafTaskLinks.size(); i++) {
				String from = leafTaskLinks.get(i);
				TaskLink tl = new TaskLink();
				tl.setFrom(from);
				tl.setTo("END");
				epmsPlan += tl + "\n";
			}
			epmsPlan += FinalTagEP.string();
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document executionPlanDoc = builder
					.parse(new java.io.ByteArrayInputStream(epmsPlan.getBytes()));
			executionPlanDoc.normalize();
			return executionPlanDoc;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	private void parse(Node node) {
		Node x, firstChild;
		String nodeName, edgeId = null;

		if (node instanceof Document) {
			Document doc = (Document) node;
			NodeList valueList = doc.getElementsByTagName("graph");
			x = (Node) valueList.item(0);
			epmsPlan += HeaderEP.string() + "\n";
			epmsPlan += Start.string() + "\n";
		} else {
			x = (Node) node;
		}

		nodeName = x.getNodeName();
		Node parentNode = (Node) x.getParentNode();
		if (nodeName.equals("node")) {
			if (x.hasAttributes()) {
				NamedNodeMap attributes = (NamedNodeMap) x.getAttributes();
				if (attributes != null)
					for (int i = 0; i < attributes.getLength(); i++) {
						Node attribute = (Node) attributes.item(i);
						if (attribute.getNodeName().equals("id")) {
							edgeId = attribute.getNodeValue();
						}
					}
			}
			if (edgeId != null) {
				nodesMapping.put(edgeId, null);
			} else {
				System.out.println("WARNING: node not mapped, id is null");
			}
		} else if (nodeName.equals("data")) {
			String valore = null;
			if (x.hasAttributes()) {
				NamedNodeMap attributes = (NamedNodeMap) x.getAttributes();
				if (attributes != null)
					for (int i = 0; i < attributes.getLength(); i++) {
						Node attribute = (Node) attributes.item(i);
						// L'attributo di 'data' è sempre 'key'
						// Ci interessa la 'key' il cui valore è 'type'
						if (attribute.getNodeValue().equals("type")) {
							firstChild = x.getFirstChild();
							valore = getValue(firstChild);
							edgeId = getParentNodeID(x);
							if (valore.equals("dataset")) {
								Dataset dataset = new Dataset();
								// Mappatura del dataset
								if (nodesMapping.containsKey(edgeId)) {
									nodesMapping.put(edgeId, dataset);
								}
							} else if (valore.equals("algorithm")) {
								DataMiningSoftware dms = new DataMiningSoftware();
								if (nodesMapping.containsKey(edgeId)) {
									nodesMapping.put(edgeId, dms);
								}
							} else if (valore.equals("model")) {
								Model model = new Model();
								if (nodesMapping.containsKey(edgeId))
									nodesMapping.put(edgeId, model);
							}
						} else if (attribute.getNodeValue().equals("location")) {
							String idOldParent, value = null;
							idOldParent = getParentNodeID(x);
							firstChild = x.getFirstChild();
							value = getValue(firstChild);
							Object o = nodesMapping.get(idOldParent);
							if (value != null) {
								if (o instanceof Model) {
									Model model = (Model) o;
									model.setLocation(value);
									nodesMapping.put(idOldParent, model);
								} else if (o instanceof Dataset) {
									Dataset dataset = (Dataset) o;
									dataset.setLocation(value);
									nodesMapping.put(idOldParent, dataset);
								}
							}
						}
					}
			}
		} else if (nodeName.equals("Dataset")) {
			String nome = null;
			String type = null;
			String href = null;
			String title = null;
			String idOldParent;
			Dataset dataset;
			if (x.hasAttributes()) {
				NamedNodeMap attributes = (NamedNodeMap) x.getAttributes();
				if (attributes != null)
					for (int i = 0; i < attributes.getLength(); i++) {
						Node attribute = (Node) attributes.item(i);
						if (attribute.getNodeName().equals("name"))
							nome = attribute.getNodeValue();
						else if (attribute.getNodeName().equals("type"))
							type = attribute.getNodeValue();
						else if (attribute.getNodeName().equals("href"))
							href = attribute.getNodeValue();
						else if (attribute.getNodeName().equals("title"))
							title = attribute.getNodeValue();
					}

			}
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof Dataset) {
				dataset = (Dataset) o;
				dataset.setName(nome);
				dataset.setType(type);
				if (href != null)
					dataset.setHref(href);
				if (title != null)
					dataset.setTitle(title);
				nodesMapping.put(idOldParent, dataset);
			}
		} else if (nodeName.equals("Access")) {
			String idOldParent;
			Dataset dataset;
			Access access = new Access();
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof Dataset) {
				dataset = (Dataset) o;
				dataset.setAccess(access);
				nodesMapping.put(idOldParent, dataset);
			}
		} else if (nodeName.equals("Location")) {
			String idOldParent = null;
			Dataset dataset;
			Access access;
			DataMiningSoftware dms;
			Usage usage;
			String value = null;
			firstChild = x.getFirstChild();
			value = getValue(firstChild);
			Location location = new Location(value);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (parentNode.getNodeName().equals("Access")) {
				if (o instanceof Dataset) {
					dataset = (Dataset) o;
					access = dataset.getAccess();
					access.setLocation(location);
					dataset.setAccess(access);
					nodesMapping.put(idOldParent, dataset);
				}
			} else if (parentNode.getNodeName().equals("Usage")) {
				if (o instanceof DataMiningSoftware) {
					dms = (DataMiningSoftware) o;
					usage = dms.getUsage();
					usage.setLocation(location);
					dms.setUsage(usage);
					nodesMapping.put(idOldParent, dms);
				}
			}
		} else if (nodeName.equals("Size")) {
			String idOldParent;
			Dataset dataset;
			Access access;
			String value = null;
			firstChild = x.getFirstChild();
			value = getValue(firstChild);
			Size size = new Size(value);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof Dataset) {
				dataset = (Dataset) o;
				access = dataset.getAccess();
				access.setSize(size);
				dataset.setAccess(access);
				nodesMapping.put(idOldParent, dataset);
			}
		} else if (nodeName.equals("Structure")) {
			String idOldParent;
			Dataset dataset;
			Structure structure = new Structure();
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof Dataset) {
				dataset = (Dataset) o;
				dataset.setStructure(structure);
				nodesMapping.put(idOldParent, dataset);
			}
		} else if (nodeName.equals("Format")) {
			String idOldParent;
			Dataset dataset;
			Structure structure;
			Format format = new Format();
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof Dataset) {
				dataset = (Dataset) o;
				structure = dataset.getStructure();
				structure.setFormat(format);
				dataset.setStructure(structure);
				nodesMapping.put(idOldParent, dataset);
			}
		} else if (nodeName.equals("AttributeSeparatorString")) {
			String idOldParent;
			Dataset dataset;
			Structure structure;
			Format format;
			firstChild = x.getFirstChild();
			String value = getValue(firstChild);
			AttributeSeparatorString ass = new AttributeSeparatorString(value);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof Dataset) {
				dataset = (Dataset) o;
				structure = dataset.getStructure();
				format = structure.getFormat();
				format.setAttributeSeparatorString(ass);
				structure.setFormat(format);
				dataset.setStructure(structure);
				nodesMapping.put(idOldParent, dataset);
			}
		} else if (nodeName.equals("RecordSeparatorString")) {
			String idOldParent;
			Dataset dataset;
			Structure structure;
			Format format;
			firstChild = x.getFirstChild();
			String value = getValue(firstChild);
			RecordSeparatorString rss = new RecordSeparatorString(value);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof Dataset) {
				dataset = (Dataset) o;
				structure = dataset.getStructure();
				format = structure.getFormat();
				format.setRecordSeparatorString(rss);
				structure.setFormat(format);
				dataset.setStructure(structure);
				nodesMapping.put(idOldParent, dataset);
			}
		} else if (nodeName.equals("UnknownTokenString")) {
			String idOldParent;
			Dataset dataset;
			Structure structure;
			Format format;
			firstChild = x.getFirstChild();
			String value = getValue(firstChild);
			UnknownTokenString uts = new UnknownTokenString(value);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof Dataset) {
				dataset = (Dataset) o;
				structure = dataset.getStructure();
				format = structure.getFormat();
				format.setUnknownTokenString(uts);
				structure.setFormat(format);
				dataset.setStructure(structure);
				nodesMapping.put(idOldParent, dataset);
			}
		} else if (nodeName.equals("Attributes")) {
			String idOldParent;
			Dataset dataset;
			Structure structure;
			Attributes attributes = new Attributes();
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof Dataset) {
				dataset = (Dataset) o;
				structure = dataset.getStructure();
				structure.setAttributes(attributes);
				dataset.setStructure(structure);
				nodesMapping.put(idOldParent, dataset);
			}
		} else if (nodeName.equals("Attribute")) {
			String nome = null;
			String type = null;
			String idOldParent;
			Dataset dataset;
			Structure structure;
			Attributes atts;
			if (x.hasAttributes()) {
				NamedNodeMap attributes = (NamedNodeMap) x.getAttributes();
				if (attributes != null)
					for (int i = 0; i < attributes.getLength(); i++) {
						Node attribute = (Node) attributes.item(i);
						if (attribute.getNodeName().equals("name"))
							nome = attribute.getNodeValue();
						else if (attribute.getNodeName().equals("type"))
							type = attribute.getNodeValue();
					}
			}
			Attribute att = new Attribute(nome, type);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof Dataset) {
				dataset = (Dataset) o;
				structure = dataset.getStructure();
				atts = structure.getAttributes();
				atts.addAttribute(att);
				structure.setAttributes(atts);
				dataset.setStructure(structure);
				nodesMapping.put(idOldParent, dataset);
			}
		} else if (nodeName.equals("SubType")) {
			String idOldParent;
			Dataset dataset;
			Structure structure;
			Attributes atts;
			String nameParent = null;
			firstChild = x.getFirstChild();
			String value = getValue(firstChild);
			SubType subType = new SubType(value);
			if (parentNode.hasAttributes()) {
				NamedNodeMap attributes = (NamedNodeMap) x.getAttributes();
				if (attributes != null)
					for (int i = 0; i < attributes.getLength(); i++) {
						Node attribute = (Node) attributes.item(i);
						if (attribute.getNodeName().equals("name"))
							nameParent = attribute.getNodeValue();
					}
			}
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof Dataset) {
				dataset = (Dataset) o;
				structure = dataset.getStructure();
				atts = structure.getAttributes();
				for (int i = 0; i < atts.size(); i++) {
					Attribute att = atts.getAttribute(i);
					if (att.getName().equals(nameParent)) {
						atts.removeAttribute(att);
						att.setSubType(subType);
						atts.addAttribute(att);
					}
				}
				structure.setAttributes(atts);
				dataset.setStructure(structure);
				nodesMapping.put(idOldParent, dataset);
			}
		} else if (nodeName.equals("Parameter")) {
			String idOldParent;
			Dataset dataset;
			Structure structure;
			Attributes atts;
			String nameParent = null;
			firstChild = x.getFirstChild();
			String value = getValue(firstChild);
			Parameter parameter = new Parameter(value);
			if (parentNode.hasAttributes()) {
				NamedNodeMap attributes = (NamedNodeMap) x.getAttributes();
				if (attributes != null)
					for (int i = 0; i < attributes.getLength(); i++) {
						Node attribute = (Node) attributes.item(i);
						if (attribute.getNodeName().equals("name"))
							nameParent = attribute.getNodeValue();
					}
			}
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof Dataset) {
				dataset = (Dataset) o;
				structure = dataset.getStructure();
				atts = structure.getAttributes();
				for (int i = 0; i < atts.size(); i++) {
					Attribute att = atts.getAttribute(i);
					if (att.getName().equals(nameParent)) {
						atts.removeAttribute(att);
						att.addParameter(parameter);
						atts.addAttribute(att);
					}
				}
				structure.setAttributes(atts);
				dataset.setStructure(structure);
				nodesMapping.put(idOldParent, dataset);
			}
		} else if (nodeName.equals("DataMiningSoftware")) {
			String nome = null;
			String href = null;
			String title = null;
			String idOldParent = null;
			DataMiningSoftware dms;
			if (x.hasAttributes()) {
				NamedNodeMap attributes = (NamedNodeMap) x.getAttributes();
				if (attributes != null)
					for (int i = 0; i < attributes.getLength(); i++) {
						Node attribute = (Node) attributes.item(i);
						if (attribute.getNodeName().equals("name"))
							nome = attribute.getNodeValue();
						if (attribute.getNodeName().equals("href"))
							href = attribute.getNodeValue();
						if (attribute.getNodeName().equals("title"))
							title = attribute.getNodeValue();
					}
			}
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof DataMiningSoftware) {
				dms = (DataMiningSoftware) o;
				dms.setName(nome);
				if (href != null)
					dms.setHref(href);
				if (title != null)
					dms.setTitle(title);
				nodesMapping.put(idOldParent, dms);
			}
		} else if (nodeName.equals("Description")) {
			String idOldParent;
			DataMiningSoftware dms;
			Description description = new Description();
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof DataMiningSoftware) {
				dms = (DataMiningSoftware) o;
				dms.setDescription(description);
				nodesMapping.put(idOldParent, dms);
			}
		} else if (nodeName.equals("KindOfData")) {
			String idOldParent = null;
			DataMiningSoftware dms;
			Description description;
			firstChild = x.getFirstChild();
			String value = getValue(firstChild);
			KindOfData kod = new KindOfData(value);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof DataMiningSoftware) {
				dms = (DataMiningSoftware) o;
				description = dms.getDescription();
				description.setKindOfData(kod);
				dms.setDescription(description);
				nodesMapping.put(idOldParent, dms);
			}
		} else if (nodeName.equals("KindOfKnowledge")) {
			String idOldParent = null;
			DataMiningSoftware dms;
			Description description;
			firstChild = x.getFirstChild();
			String value = getValue(firstChild);
			KindOfKnowledge kok = new KindOfKnowledge(value);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof DataMiningSoftware) {
				dms = (DataMiningSoftware) o;
				description = dms.getDescription();
				description.setKindOfKnowledge(kok);
				dms.setDescription(description);
				nodesMapping.put(idOldParent, dms);
			}
		} else if (nodeName.equals("KindOfTechnique")) {
			String idOldParent = null;
			DataMiningSoftware dms;
			Description description;
			firstChild = x.getFirstChild();
			String value = getValue(firstChild);
			KindOfTechnique kot = new KindOfTechnique(value);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof DataMiningSoftware) {
				dms = (DataMiningSoftware) o;
				description = dms.getDescription();
				description.setKindOfTechnique(kot);
				dms.setDescription(description);
				nodesMapping.put(idOldParent, dms);
			}
		} else if (nodeName.equals("DrivingMethod")) {
			String idOldParent = null;
			DataMiningSoftware dms;
			Description description;
			firstChild = x.getFirstChild();
			String value = getValue(firstChild);
			DrivingMethod dm = new DrivingMethod(value);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof DataMiningSoftware) {
				dms = (DataMiningSoftware) o;
				description = dms.getDescription();
				description.setDrivingMethod(dm);
				dms.setDescription(description);
				nodesMapping.put(idOldParent, dms);
			}
		} else if (nodeName.equals("Usage")) {
			String idOldParent;
			DataMiningSoftware dms;
			Usage usage = new Usage();
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof DataMiningSoftware) {
				dms = (DataMiningSoftware) o;
				dms.setUsage(usage);
				nodesMapping.put(idOldParent, dms);
			}
		} else if (nodeName.equals("KindOfSoftware")) {
			String idOldParent = null;
			DataMiningSoftware dms;
			Usage usage;
			firstChild = x.getFirstChild();
			String value = getValue(firstChild);
			KindOfSoftware kos = new KindOfSoftware(value);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof DataMiningSoftware) {
				dms = (DataMiningSoftware) o;
				usage = dms.getUsage();
				usage.setKindOfSoftware(kos);
				dms.setUsage(usage);
				nodesMapping.put(idOldParent, dms);
			}
		} else if (nodeName.equals("Invocation")) {
			String idOldParent = null;
			DataMiningSoftware dms;
			Usage usage;
			firstChild = x.getFirstChild();
			String value = getValue(firstChild);
			Invocation inv = new Invocation(value);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof DataMiningSoftware) {
				dms = (DataMiningSoftware) o;
				usage = dms.getUsage();
				usage.setInvocation(inv);
				dms.setUsage(usage);
				nodesMapping.put(idOldParent, dms);
			}
		} else if (nodeName.equals("ManualPath")) {
			String idOldParent = null;
			DataMiningSoftware dms;
			Usage usage;
			firstChild = x.getFirstChild();
			String value = getValue(firstChild);
			ManualPath mp = new ManualPath(value);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof DataMiningSoftware) {
				dms = (DataMiningSoftware) o;
				usage = dms.getUsage();
				usage.setManualPath(mp);
				dms.setUsage(usage);
				nodesMapping.put(idOldParent, dms);
			}
		} else if (nodeName.equals("OutputFolder")) {
			String idOldParent = null;
			DataMiningSoftware dms;
			Usage usage;
			firstChild = x.getFirstChild();
			String value = getValue(firstChild);
			OutputFolder outF = new OutputFolder(value);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof DataMiningSoftware) {
				dms = (DataMiningSoftware) o;
				usage = dms.getUsage();
				usage.setOutputFolder(outF);
				dms.setUsage(usage);
				nodesMapping.put(idOldParent, dms);
			}
		} else if (nodeName.equals("Args")) {
			String idOldParent;
			DataMiningSoftware dms;
			Usage usage;
			Args args = new Args();
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof DataMiningSoftware) {
				dms = (DataMiningSoftware) o;
				usage = dms.getUsage();
				usage.setArgs(args);
				dms.setUsage(usage);
				nodesMapping.put(idOldParent, dms);
			}
		} else if (nodeName.equals("Arg")) {
			String idOldParent = null;
			String nome = null;
			DataMiningSoftware dms;
			Usage usage;
			boolean stop = false;
			Arg arg = null;
			Args args;
			if (x.hasAttributes()) {
				NamedNodeMap attributes = (NamedNodeMap) x.getAttributes();
				if (attributes != null)
					for (int i = 0; i < attributes.getLength() && !stop; i++) {
						Node attribute = (Node) attributes.item(i);
						if (attribute.getNodeName().equals("name")) {
							nome = attribute.getNodeValue();
							arg = new Arg(nome);
							stop = true;
						}
					}
				if (arg != null) {
					for (int i = 0; i < attributes.getLength(); i++) {
						Node attribute = (Node) attributes.item(i);
						if (attribute.getNodeName().equals("type"))
							arg.setType(attribute.getNodeValue());
						else if (attribute.getNodeName().equals("default"))
							arg.setDefault(attribute.getNodeValue());
						else if (attribute.getNodeName().equals("mandatory"))
							arg.setMandatory(attribute.getNodeValue());
					}
				}
			}
			firstChild = x.getFirstChild();
			String value = getValue(firstChild);
			arg.setValue(value);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof DataMiningSoftware) {
				dms = (DataMiningSoftware) o;
				usage = dms.getUsage();
				args = usage.getArgs();
				args.addArg(arg);
				usage.setArgs(args);
				dms.setUsage(usage);
				nodesMapping.put(idOldParent, dms);
			}
		} else if (nodeName.equals("HostName")) {
			String idOldParent = null;
			DataMiningSoftware dms;
			Usage usage;
			HostName hostName = new HostName();
			firstChild = x.getFirstChild();
			String value = getValue(firstChild);
			hostName.setHostName(value);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof DataMiningSoftware) {
				dms = (DataMiningSoftware) o;
				usage = dms.getUsage();
				usage.setHostName(hostName);
				dms.setUsage(usage);
				nodesMapping.put(idOldParent, dms);
			}
		} else if (nodeName.equals("DocumentationURL")) {
			String idOldParent = null;
			DataMiningSoftware dms;
			Usage usage;
			DocumentationURL docURL = new DocumentationURL();
			firstChild = x.getFirstChild();
			String value = getValue(firstChild);
			docURL.setDocumentationURL(value);
			idOldParent = getParentNodeID(x);
			Object o = nodesMapping.get(idOldParent);
			if (o instanceof DataMiningSoftware) {
				dms = (DataMiningSoftware) o;
				usage = dms.getUsage();
				usage.setDocumentationURL(docURL);
				dms.setUsage(usage);
				nodesMapping.put(idOldParent, dms);
			}
		} else if (nodeName.equals("edge")) {
			String sourceId = null;
			String targetId = null;
			Dataset dataset = null;
			Model model = null;
			DataMiningSoftware dms = null;
			if (x.hasAttributes()) {
				NamedNodeMap attributes = (NamedNodeMap) x.getAttributes();
				if (attributes != null) {
					for (int i = 0; i < attributes.getLength(); i++) {
						Node attribute = (Node) attributes.item(i);
						if (attribute.getNodeName().equals("id")) {
							edgeId = attribute.getNodeValue();
						} else if (attribute.getNodeName().equals("source")) {
							sourceId = attribute.getNodeValue();
						} else if (attribute.getNodeName().equals("target")) {
							targetId = attribute.getNodeValue();
						}
					}
				}
			} else {
				System.out.println("#ERROR: edge with no attributes");
				System.exit(-1);
			}
			Object sourceObj = nodesMapping.get(sourceId);
			Object targetObj = nodesMapping.get(targetId);
			Edge edge = new Edge(edgeId, sourceId, targetId);
			edges.add(edge);
			if ((sourceObj instanceof DMSoftInput)
					&& (targetObj instanceof DataMiningSoftware)) {
				/*
				 * Edge connecting a Dataset or a Model to a tool or an
				 * algorithm
				 */
				String nameSource = null;
				if (sourceObj instanceof Model) {
					model = (Model) sourceObj;
					nameSource = model.getName();
				} else if (sourceObj instanceof Dataset) {
					dataset = (Dataset) sourceObj;
					nameSource = dataset.getName();
				}
				dms = (DataMiningSoftware) targetObj;
				Input input = new Input();
				/*
				 * Per verificare l'attributo: Idref: Verifico se l'input è:
				 * a)l'output di un algoritmo OPPURE b) un input già creato in
				 * precedenza
				 */
				boolean stop = false;
				/*
				 * a) We check whether an edge having the source of the current
				 * edge as target exists
				 */
				for (Edge parentEdge : edges) {
					if (parentEdge.getTarget().equals(sourceId)) {
						/*
						 * Ora bisogna trovare il TASK che contiene l'edge
						 * tovato Cioè il TASK che ha lo stesso ID dell'Edge
						 * PAdre (cioè targetPadre==Source_Edge-trovato)(bisogna
						 * risalire l'albero)
						 */
						String parentSourceId = parentEdge.getSource();
						for (Edge grandParentEdge : edges) {
							if (grandParentEdge.getTarget().equals(
									parentSourceId)) {
								Task aus2 = (Task) nodesMapping
										.get(grandParentEdge.getID());
								Computation computation = aus2.getComputation();
								for (int k = 0; k < computation.sizeOutput(); k++) {
									Output output = computation.getOutput(k);
									Dataset datasetTemp = output.getDataset();
									if (dataset != null
											&& (datasetTemp.equals(dataset))) {
										input.setIdref(output.getId());
										stop = true;
									} else if (datasetTemp.getName().equals(
											nameSource)) {
										input.setIdref(output.getId());
										stop = true;
									}
								}
							}
						}
					}
				}
				// b) Verifico nei Task creati se esiste un input uguale
				if (!stop) {
					for (String taskID : listTaskID) {
						Task taskTemp = (Task) nodesMapping.get(taskID);
						Computation computationTemp = taskTemp.getComputation();
						for (int j = 0; j < computationTemp.sizeInput(); j++) {
							Input inputTemp = computationTemp.getInput(j);
							if (inputTemp.getId() != null) {
								Dataset datasetTemp = inputTemp.getDataset();
								if (dataset != null
										&& (datasetTemp.equals(dataset))) {
									input.setIdref(inputTemp.getId());
									stop = true;
								} else {
									// there is a model in input
									if (dataset == null
											&& datasetTemp.getName().equals(
													nameSource)) {
										input.setIdref(inputTemp.getId());
										stop = true;
									}
								}
							}
						}
					}
				}
				/*
				 * Ora verifico se esiste una computazione con questo algoritmo.
				 * In caso di esito positivo aggiungo il nuovo input alla
				 * computazione. Devo quindi verificare nei Task che hanno lo
				 * stesso ID dei rami con Target uguale al Target del ramo in
				 * questione (cioè che si collegano a questo algoritmo)
				 */
				Task task = null;
				stop = false;
				String id2 = null;
				for (int i = 0; i < edges.size() && !stop; i++) {
					Edge edgeTemp1 = edges.get(i);
					if (edgeTemp1.getTarget().equals(targetId)) {
						id2 = edgeTemp1.getID();
						task = (Task) nodesMapping.get(id2);
						stop = true;
					}
				}
				if (task != null) {
					// A computation with this algorithm exists
					Computation computation = task.getComputation();
					computation.addInput(input);
					Program program = computation.getProgram();
					DataMiningSoftware dmsTemp = program
							.getDataMiningSoftware();
					Usage usage = dmsTemp.getUsage();
					Args args = null;
					if (usage == null) {
						usage = new Usage();
						args = new Args();
					} else {
						args = usage.getArgs();
					}
					String value = null;
					String value2 = null;
					Node firstChildEdge = null;
					NodeList childEdge = (NodeList) x.getChildNodes();
					if (childEdge != null) {
						for (int i = 0; i < childEdge.getLength(); i++) {
							if (childEdge.item(i).getNodeType() == Node.ELEMENT_NODE) {
								firstChildEdge = childEdge.item(i);
							}
						}
					}
					if (firstChildEdge == null) {
						System.out
								.println("ERRORE in '"
										+ edgeId
										+ "' : Indicare il tipo di file di input che si vuole ottenere!!!");
						System.out.println("ERRORE in '" + edgeId
								+ "' : Rivedere il modello concettuale");
						System.exit(-1);
					}
					firstChild = firstChildEdge.getFirstChild();
					if (firstChild != null) {
						if (firstChild.getNodeType() == Node.TEXT_NODE) {
							if (firstChild.getNodeValue().codePointAt(0) != 10) {
								value = firstChild.getNodeValue();
								value2 = EdgeMapping.getInputValue(value);
								if (value2 == null) {
									System.out
											.println("ERRORE in '"
													+ edgeId
													+ "' : Tipo file di input errato!!!");
									System.out
											.println("ERRORE in '"
													+ edgeId
													+ "' : Rivedere il modello concettuale");
									System.exit(-1);
								}
								Arg arg = new Arg(value2);
								arg.setValue(input.getIdref());
								if (args == null)
									args = new Args();
								args.addArg(arg);
								usage.setArgs(args);
								dmsTemp.setUsage(usage);
								program.setDataMiningSoftware(dmsTemp);
								computation.setProgram(program);
							}
						}
					}
					task.setComputation(computation);
					nodesMapping.put(id2, task);
					if (!mappedNodes.contains(sourceId)) {
						mappedNodes.add(sourceId);
						Vector<String> listTaskNode = mapTaskNode.get(task
								.getName());
						listTaskNode.add(sourceId);
						mapTaskNode.put(task.getName(), listTaskNode);
					}
					if (!listTaskID.contains(id2)) {
						listTaskID.add(id2);
					}
					/*
					 * In order to create the tasklink I must find the edge
					 * which has as target the source of the current edge and
					 * then identify to which task it belongs.
					 */
					String from = null;
					stop = false;
					for (int i = 0; i < edges.size() && !stop; i++) {
						Edge edge2 = edges.get(i);
						if (edge2.getTarget().equals(sourceId)) {
							/*
							 * L'edge trovato apparterrà al task che ha lo
							 * stesso ID dell'Edge PAdre(cioè
							 * targetPadre==Source_Edge-trovato)
							 */
							for (int j = 0; j < edges.size() && !stop; j++) {
								Edge edgePadre = edges.get(j);
								if (edge2.getSource().equals(
										edgePadre.getTarget())) {
									Task aus2 = (Task) nodesMapping
											.get(edgePadre.getID());
									from = aus2.getName();
									stop = true;
								}
							}
						}
					}
					TaskLink taskLink = new TaskLink();
					if (from != null) {
						taskLink.setFrom(from);
					} else {
						/*
						 * TODO This workaround may not work when the workflow
						 * has multiple root nodes
						 */
						taskLink.setFrom("START");
					}
					taskLink.setTo(task.getName());
					// ****
					String keyEdge = null;
					if (input.getIdref() != null) {
						keyEdge = task.getName() + ":" + input.getIdref();
					} else {
						keyEdge = task.getName() + ":" + input.getId();
					}
					infoEdge.put(keyEdge, edge);
					// ****
					Integer count = null;
					if (!transferCountInput.containsKey(task.getName())) {
						count = new Integer(0);
						transferCountInput.put(task.getName(), count);
					} else {
						count = transferCountInput.get(task.getName());
						int valueCount = count.intValue();
						valueCount++;
						count = new Integer(valueCount);
						transferCountInput.put(task.getName(), count);
					}
					String nameRis_PRE = task.getName() + "_inputTransfer";
					String nameRis_POST = nameRis_PRE + count.intValue();
					if (input.getIdref() != null) {
						keyEdge = nameRis_POST + ":" + input.getIdref();
					} else {
						keyEdge = nameRis_POST + ":" + input.getId();
					}
					// System.out.println("====>keyEdge="+keyEdge);
					infoEdge.put(keyEdge, edge);
					if (!taskLinks.contains(taskLink)) {
						taskLinks.add(taskLink);
					}
				} else {
					/*
					 * task == null: a computation with this algorithm does not
					 * exist, hence we create a new computation
					 */
					if (input.getIdref() == null) {
						numInput++;
						String inputName = "input" + numInput;
						input.setId(inputName);
						if (dataset != null) {
							input.setDataset(dataset);
							if (dataset.getHref() != null) {
								input.setHref(dataset.getHref());
							}
							if (dataset.getTitle() != null) {
								input.setTitle(dataset.getTitle());
							}
						}
					}
					numProgram++;
					String nomeProgram = "program" + numProgram;
					Program program = new Program(nomeProgram);
					Usage usage = dms.getUsage();
					Args args = null;
					if (usage == null) {
						usage = new Usage();
						args = new Args();
					} else {
						args = usage.getArgs();
					}
					String value, value2 = null;
					Node firstChildEdge = null;
					NodeList childEdge = (NodeList) x.getChildNodes();
					if (childEdge != null) {
						for (int i = 0; i < childEdge.getLength(); i++) {
							if (childEdge.item(i).getNodeType() == Node.ELEMENT_NODE) {
								firstChildEdge = childEdge.item(i);
							}
						}
					}
					if (firstChildEdge == null) {
						System.out.println("ERROR in '" + edgeId
								+ "' : input file type missing");
						System.exit(-1);
					}
					firstChild = firstChildEdge.getFirstChild();
					if (firstChild != null) {
						if (firstChild.getNodeType() == Node.TEXT_NODE) {
							if (firstChild.getNodeValue().codePointAt(0) != 10) {
								value = firstChild.getNodeValue();
								value2 = EdgeMapping.getInputValue(value);
								if (value2 == null) {
									System.out
											.println("ERRORE in '"
													+ edgeId
													+ "' : Tipo file di input errato!!!");
									System.out
											.println("ERRORE in '"
													+ edgeId
													+ "' : Rivedere il modello concettuale");
									System.exit(-1);
								}
								Arg arg = new Arg(value2);
								if (input.getIdref() == null) {
									arg.setValue(input.getId());
								} else {
									arg.setValue(input.getIdref());
								}
								if (args == null)
									args = new Args();
								args.addArg(arg);
								usage.setArgs(args);
								dms.setUsage(usage);
							}
						}
					}
					program.setDataMiningSoftware(dms);
					if (dms.getHref() != null) {
						program.setHref(dms.getHref());
					}
					if (dms.getTitle() != null) {
						program.setTitle(dms.getTitle());
					}
					Computation computation = new Computation();
					computation.addInput(input);
					computation.setProgram(program);
					numTask++;
					String taskName = "sub_job" + numTask;
					task = new Task(taskName);
					task.setComputation(computation);
					nodesMapping.put(edgeId, task);
					Vector<String> listTaskNode = new Vector<String>();
					if (!mappedNodes.contains(sourceId)) {
						mappedNodes.add(sourceId);
						listTaskNode.add(sourceId);
					}
					if (!mappedNodes.contains(targetId)) {
						mappedNodes.add(targetId);
						listTaskNode.add(targetId);
					}
					mapTaskNode.put(task.getName(), listTaskNode);
					if (!listTaskID.contains(edgeId)) {
						listTaskID.add(edgeId);
					}

					// ****
					String keyEdge = null;
					if (input.getIdref() != null) {
						keyEdge = task.getName() + ":" + input.getIdref();
					} else {
						keyEdge = task.getName() + ":" + input.getId();
					}
					infoEdge.put(keyEdge, edge);
					// ****
					Integer count = null;
					if (!transferCountInput.containsKey(task.getName())) {
						count = new Integer(0);
						transferCountInput.put(task.getName(), count);
					} else {
						count = transferCountInput.get(task.getName());
						int valueCount = count.intValue();
						valueCount++;
						count = new Integer(valueCount);
						transferCountInput.put(task.getName(), count);
					}
					String nameRis_PRE = task.getName() + "_inputTransfer";
					String nameRis_POST = nameRis_PRE + count.intValue();
					if (input.getIdref() != null) {
						keyEdge = nameRis_POST + ":" + input.getIdref();
					} else {
						keyEdge = nameRis_POST + ":" + input.getId();
					}
					// System.out.println("====>keyEdge="+keyEdge);
					infoEdge.put(keyEdge, edge);
					// ******

					// TaskLinks creation
					TaskLink taskLink = new TaskLink();
					if (taskLinks.size() == 0) {
						taskLink.setFrom("START");
						taskLink.setTo(taskName);
					} else {
						String from = null;
						// Devo trovare l'Edge ke ha come Target la Source del
						// Edge in questione,
						// e individuare a quale task appartiene
						stop = false;
						for (int i = 0; i < edges.size() && !stop; i++) {
							Edge edgeTemp1 = edges.get(i);
							if (edgeTemp1.getTarget().equals(sourceId)) {
								// L'edge trovato apparterrà al task ke ha lo
								// stesso ID
								// dell'Edge PAdre(cioè
								// target==Source_Edge-trovato)
								for (int j = 0; j < edges.size() && !stop; j++) {
									Edge edgePadre = edges.get(j);
									if (edgeTemp1.getSource().equals(
											edgePadre.getTarget())) {
										Task aus2 = (Task) nodesMapping
												.get(edgePadre.getID());
										from = aus2.getName();
										stop = true;
									}
								}
							}
						}
						if (from != null) {
							taskLink.setFrom(from);
						} else {
							taskLink.setFrom("START");
						}
						taskLink.setTo(taskName);
					}
					taskLinks.add(taskLink);
				}
			} else if ((sourceObj instanceof DataMiningSoftware)
					&& (targetObj instanceof DMSoftInput)) {
				/*
				 * Edge connecting a tool or an algorithm to a Dataset or a
				 * Model
				 */

				// Devo trovare quel Task che usa l'algoritmo contenuto in
				// ValSource
				// ==>Il task ha lo stesso ID di un certo Edge che ha come
				// Target la Source dell'Edge in questione
				String id2 = null;
				dms = (DataMiningSoftware) sourceObj;
				if (targetObj instanceof Model) {
					model = (Model) targetObj;
				} else if (targetObj instanceof Dataset) {
					dataset = (Dataset) targetObj;
				}
				// FIXME ### Sometimes it looks for edges which have not been
				// added yet: it happens when the current edge's id number is
				// less than the edge's id number it is looking for
				for (int i = 0; i < edges.size(); i++) {
					Edge edgeTemp = edges.get(i);
					if (edgeTemp.getTarget().equals(sourceId)) {
						Object o = nodesMapping.get(edgeTemp.getID());
						/*
						 * Verifico se a questo ID corrisponde un TASK perché
						 * potrebbe capitare il caso in cui un Task ha molti
						 * input allora bisogna trovare quell'Edge che ha dato
						 * l'ID al TASK perché un solo input darà l'ID al task
						 */
						if (o instanceof Task) {
							id2 = edgeTemp.getID();
							break;
						}
					}
				}
				Task task = (Task) nodesMapping.get(id2);
				numOutput++;
				String nomeOutput = "output" + numOutput;
				Output output = new Output(nomeOutput);
				Computation computation = task.getComputation();
				// aggiungo un argomento di output al program
				Program program = computation.getProgram();
				DataMiningSoftware dmsTemp = program.getDataMiningSoftware();
				Usage usage = dmsTemp.getUsage();
				Args args = null;
				if (usage == null) {
					usage = new Usage();
					args = new Args();
				} else {
					args = usage.getArgs();
				}
				String value = null;
				String value2 = null;
				Node firstChildEdge = null;
				NodeList childEdge = (NodeList) x.getChildNodes();
				if (childEdge != null)
					for (int i = 0; i < childEdge.getLength(); i++)
						if (childEdge.item(i).getNodeType() == Node.ELEMENT_NODE)
							firstChildEdge = childEdge.item(i);
				if (firstChildEdge == null) {
					System.out
							.println("ERRORE in '"
									+ edgeId
									+ "' : Indicare il tipo di file di output che si vuole ottenere!!!");
					System.out.println("ERRORE in '" + edgeId
							+ "' : Rivedere il modello concettuale");
					System.exit(-1);
				}
				firstChild = firstChildEdge.getFirstChild();
				if (firstChild != null) {
					if (firstChild.getNodeType() == Node.TEXT_NODE) {
						if (firstChild.getNodeValue().codePointAt(0) != 10) {
							value = firstChild.getNodeValue();
							value2 = EdgeMapping.getOutputValue(value);
							if (value2 == null) {
								System.out.println("ERRORE in '" + edgeId
										+ "' : Tipo file di output errato!!!");
								System.out
										.println("ERRORE in '"
												+ edgeId
												+ "' : Rivedere il modello concettuale");
								System.exit(-1);
							}
							Arg arg = new Arg(value2);
							arg.setValue(output.getId());
							if (args == null)
								args = new Args();
							args.addArg(arg);
							usage.setArgs(args);
							dmsTemp.setUsage(usage);
							program.setDataMiningSoftware(dmsTemp);
							computation.setProgram(program);
						}
					}
				}
				String datasetOutName = null;
				if (value2.equals("OutputDataSet"))
					datasetOutName = task.getName() + "_" + nomeOutput
							+ ".arff";
				else if (value2.equals("OutputModel")) {
					// TODO (-4-) EPMS: change to a unique name
					datasetOutName = task.getName() + "_" + "model.model";
				}
				String location = null;
				Dataset datasetAUS = new Dataset();
				if (dataset == null && model != null) {
					model.setName(datasetOutName);
					location = model.getLocation();
					nodesMapping.put(targetId, model);
				} else if (dataset != null && model == null) {
					dataset.setName(datasetOutName);
					location = dataset.getLocation();
					nodesMapping.put(targetId, dataset);
				}
				datasetAUS.setName(datasetOutName);
				output.setDataset(datasetAUS);
				computation.addOutput(output);
				task.setComputation(computation);
				nodesMapping.put(id2, task);
				if (!listTaskID.contains(id2))
					listTaskID.add(id2);
				if ((!location.equals("")) && (location != null)) {
					listEdgeTransfer.add(edge.getID());
					String from = task.getName();
					String To = null;
					numTransfer++;
					String nomeTransfer = "Transfer" + numTransfer;
					To = nomeTransfer;
					Task taskTransfer = new Task(nomeTransfer);
					DataTransfer dataTransfer = new DataTransfer();
					dataTransfer.setDest(location);
					Input input = new Input();
					input.setIdref(output.getId());
					dataTransfer.setInput(input);
					taskTransfer.setDataTransfer(dataTransfer);
					nodesMapping.put(edgeId, nomeTransfer);
					nodesMapping.put(nomeTransfer, taskTransfer);
					if (!mappedNodes.contains(targetId)) {
						mappedNodes.add(targetId);
						Vector<String> listTaskNode = new Vector<String>();
						listTaskNode.add(targetId);
						mapTaskNode.put(nomeTransfer, listTaskNode);
					}
					if (!listTaskID.contains(nomeTransfer))
						listTaskID.add(nomeTransfer);
					TaskLink taskLink = new TaskLink();
					taskLink.setFrom(from);
					taskLink.setTo(To);
					taskLinks.add(taskLink);
					// ****
					String keyEdge = taskTransfer.getName() + ":"
							+ output.getId();
					infoEdge.put(keyEdge, edge);
					// ****
				} else {
					Integer count = null;
					if (!transferCount.containsKey(task.getName())) {
						count = new Integer(0);
						transferCount.put(task.getName(), count);
					} else {
						count = transferCount.get(task.getName());
						int valueCount = count.intValue();
						valueCount++;
						count = new Integer(valueCount);
						transferCount.put(task.getName(), count);
					}
					// TODO (-4-) EPMS: change to a unique name
					String nameRis_PRE = task.getName() + "_resultTransfer";
					String nameRis_POST = nameRis_PRE + count.intValue();
					if (!nodesMapping.containsKey(nameRis_POST)) {
						Vector<String> listTaskNode = new Vector<String>();
						listTaskNode.add(targetId);
						mapTaskNode.put(nameRis_POST, listTaskNode);
					} else {
						Vector<String> listTaskNode = mapTaskNode
								.get(nameRis_POST);
						listTaskNode.add(targetId);
						mapTaskNode.put(nameRis_POST, listTaskNode);
					}
					// ****
					String keyEdge = nameRis_POST + ":" + output.getId();
					infoEdge.put(keyEdge, edge);
					// ****
				}
				// listTaskNode.add(target);
				// mapTaskNode.put(task.getName(),listTaskNode);
			} else if ((sourceObj instanceof DMSoftInput)
					&& (targetObj instanceof DMSoftInput)) {
				// TRANSFER
				listEdgeTransfer.add(edge.getID());
				DMSoftInput resultsSource = (DMSoftInput) sourceObj;
				DMSoftInput resultsTarget = (DMSoftInput) targetObj;
				String from = null;
				String To = null;
				numTransfer++;
				String nomeTransfer = "Transfer" + numTransfer;
				To = nomeTransfer;
				Task task = new Task(nomeTransfer);
				DataTransfer dataTransfer = new DataTransfer();
				dataTransfer.setDest(resultsTarget.getLocation());
				Input input = new Input();
				// Devo trovare il task di appartenenza della sorgente,e quindi
				// ottenere il nome dell'output
				boolean stop = false;
				for (int i = 0; i < edges.size() && !stop; i++) {
					Edge edgeTemp = edges.get(i);

					// Ricerca del ramo precedente nel grafo
					if (edgeTemp.getTarget().equals(sourceId)) {
						if (listEdgeTransfer.contains(edgeTemp.getID())) {
							String nomeTransferPadre = (String) nodesMapping
									.get(edgeTemp.getID());
							Task aus2 = (Task) nodesMapping
									.get(nomeTransferPadre);
							from = aus2.getName();
							DataTransfer dataTransferTemp = (DataTransfer) aus2
									.getDataTransfer();
							Input inputTemp = null;
							if (dataTransferTemp != null) {
								inputTemp = dataTransferTemp.getInput();
								if (inputTemp != null)
									input.setIdref(inputTemp.getIdref());
								stop = true;
							}
						}
						// L'edge trovato apparterrà al task ke ha lo stesso ID
						// dell'Edge PAdre(cioè
						// targetPadre==Source_Edge-trovato)
						for (int j = 0; j < edges.size() && !stop; j++) {
							Edge edgePadre = edges.get(j);
							if (edgeTemp.getSource().equals(
									edgePadre.getTarget())) {
								Task aus2 = (Task) nodesMapping.get(edgePadre
										.getID());
								from = aus2.getName();
								// System.out.println("EDge id="+edge.getID()+"FROM="+from);
								Computation computation = aus2.getComputation();
								for (int l = 0; l < computation.sizeOutput(); l++) {
									Output output = computation.getOutput(l);
									Dataset datasetTemp = output.getDataset();
									if (datasetTemp.getName().equals(
											resultsSource.getName())) {
										input.setIdref(output.getId());
										stop = true;
									}
									if (sourceObj instanceof Dataset)
										dataset = (Dataset) sourceObj;
									if (dataset != null
											&& (datasetTemp.equals(dataset))) {
										input.setIdref(output.getId());
										stop = true;
									} else if (datasetTemp.getName().equals(
											resultsSource.getName())) {
										input.setIdref(output.getId());
										stop = true;
									}
								}
							}
						}
					}
				}
				if (stop == false) {
					// la sorgente non appartiene a nessun Task; si tratta di un
					// Transfer indipendente
					// Un transfer indipendente ha bisogno di essere specificato
					// concretamente
					if (sourceObj instanceof Dataset) {
						dataset = (Dataset) sourceObj;
						if (dataset.getHref() != null)
							input.setHref(dataset.getHref());
						// if(dataset.getTitle()!=null)
						// input.setTitle(dataset.getTitle());
					} else
						System.out
								.println("EPMSTranslate: ERRORE, la sorgente del Transfer deve essere un Dataset");
				}
				dataTransfer.setInput(input);
				task.setDataTransfer(dataTransfer);
				nodesMapping.put(edgeId, nomeTransfer);
				nodesMapping.put(nomeTransfer, task);
				if (!mappedNodes.contains(targetId)) {
					mappedNodes.add(targetId);
					Vector<String> listTaskNode = new Vector<String>();
					listTaskNode.add(targetId);
					mapTaskNode.put(nomeTransfer, listTaskNode);
				}
				if (!mappedNodes.contains(sourceId)) {
					mappedNodes.add(sourceId);
					Vector<String> listTaskNode = mapTaskNode.get(nomeTransfer);
					if (listTaskNode == null)
						listTaskNode = new Vector<String>();
					listTaskNode.add(sourceId);
					mapTaskNode.put(nomeTransfer, listTaskNode);
				}
				if (!listTaskID.contains(nomeTransfer))
					listTaskID.add(nomeTransfer);
				TaskLink taskLink = new TaskLink();
				if (from != null) {
					taskLink.setFrom(from);
				} else {
					taskLink.setFrom("START");
				}
				taskLink.setTo(To);
				taskLinks.add(taskLink);
				// ****
				String keyEdge = null;
				if (input.getIdref() != null) {
					keyEdge = task.getName() + ":" + input.getIdref();
				} else if (!input.getHref().equals("")) {
					keyEdge = task.getName() + ":" + input.getHref();
				}
				infoEdge.put(keyEdge, edge);
				// ****
			}
		}
		NodeList children = (NodeList) x.getChildNodes();
		if (!nodeName.equals("edge") && children != null) {
			for (int i = 0; i < children.getLength(); i++) {
				if (children.item(i).getNodeType() == Node.ELEMENT_NODE) {
					parse(children.item(i));
				}
			}
		}
	}

	private String getValue(Node firstChild) {
		if (firstChild != null) {
			if (firstChild.getNodeType() == Node.TEXT_NODE) {
				if (firstChild.getNodeValue().codePointAt(0) != 10) {
					return firstChild.getNodeValue();
				}
			}
		}
		return null;
	}

	// Metodo il cui compito è quello di restituire l'ID del tag Node, padre
	// dell'oggetto node passato come riferimento
	private String getParentNodeID(Node node) {
		if (node.getNodeName().equals("node")) {
			if (node.hasAttributes()) {
				NamedNodeMap attributes = (NamedNodeMap) node.getAttributes();
				if (attributes != null) {
					for (int i = 0; i < attributes.getLength(); i++) {
						Node attribute = (Node) attributes.item(i);
						if (attribute.getNodeName().equals("id")) {
							return attribute.getNodeValue();
						}
					}
				}
			}
		}
		return getParentNodeID(node.getParentNode());
	}

	public HashMap<String, Vector> getTaskNodesMapping() {
		return mapTaskNode;
	}

	public HashMap<String, Object> getIdNodeMapping() {
		return nodesMapping;
	}

	public HashMap<String, Edge> getInfoEdge() {
		return infoEdge;
	}

}
